# Submitting with Git



## Submitting when a  repo is provided



As we will use existing repositories for most of the tasks in this module, we will need to know how to **fork** the supplied project.

A fork is a copy of a project that allows you to make changes without actually changing the original project. This makes it a good way of taking a project as a starting point to then go off and do your own thing.



You need to be logged in to see the ```fork``` option. If you navigate to a project (for example, by using a url) you should see a page with this layout. Ensure you are on the 'project overview' available on the left.

![gitlab page](git_resources/gitlab1.png)



The fork button is found in the top right of the pages main body:

![top right of the pages main body](git_resources/gitlab2.png)



**When GitLab asks what namespace you wish to add the repo to, select your user account.**



Once you have forked the project you should notice that you now have a repo on your GitLab account (where your-username represents your personal gitlab username):

https://gitlab.uwe.ac.uk/your-username/example.git

With a version of the repo forked for your own use, you can clone the repo and edit it locally.



Cloning a repo makes a copy of a repo where edits can be easily synchronized with the remote repository. The intention with a clone is that all clones remain the same (allowing for working on multiple machines, sharing code and backing up online).

To clone obtain the git url from you GitLab project by clicking the clone button and copying the url.

![gitlab clone button shows url](git_resources/git_clone.png)

Use that url in the following command:

```
git clone https://gitlab.uwe.ac.uk/USER_NAME/PROJECT_NAME.git
```



From here you can work with git as usual. e.g:

```
cd example

touch README.md

git add README.md

git commit -m "added README"

git push 

```



When submitting your repo, just check that everything is visible in GitLab and as expected and submit the URL to your repo (ensuring that you have set the permissions to share with us).



## Submitting with no supplied repo



If a repo is not supplied for a worksheet, you should create a ```cpp``` file with the challenge name in your project directory, using the form:

```
touch task1.cpp
```

This creates a file named 'task.cpp' in your current directory.

To submit this work, you should ```add``` and ```commit``` the file to a git project and share the link to the repository for it to be marked.



Overall, this may look something like this:

>  **First, make a project on gitlab**.

```bash
cd Documents/programming-in-cpp

#clone the project you made
git clone https://gitlab.uwe.ac.uk/USER_NAME/PROJECT_NAME.git

# add files required
touch task1.cpp
touch task2.cpp
git add task1.cpp task2.cpp

# Code your solutions 
code .
# Probably alot of other stuff here...

# Save your solutions
git commit "There should often be multiple commits that describe what you did"

# Push your solution to the server
git push
```

> To finish, submit the URL to your repo on BlackBoard and ensure it is shared with your tutors.



If you are feeling confident, for larger projects (such as the assignments), you a free to use your own structure. We'd recommend using a ```main.cpp``` as your entry point.



When submitting your repo, just check that everything is visible in GitLab and as expected and submit the URL to your repo (ensuring that you have set the permissions to share with us).